#include "stdafx.h"

/*
    BootRes: Breaks apart and create the activity.bmp file in bootres.dll
    Copyright (C) 2010  Jeffrey Bush <jeff@coderforlife.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/gpl.html>.
*/

#define ANIM_WIDTH	200
#define ANIM_HEIGHT	200
#define FRAMES		105

const __int64 creation_time		= 0x01C9EA0A584DA403; //6/10/2009 1:30:50 PM
const __int64 modification_time	= 0x01C9EA0A586EF727; //6/10/2009 1:30:51 PM
const __int64 accessed_time		= 0x01C9EA0A584DA403; //6/10/2009 1:30:50 PM

using namespace System;
using namespace System::IO;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;

String ^getExt(ImageFormat ^format) {
	if (format == format->Png) {
		return L".png";
	} else if (format == format->Gif) {
		return L".gif";
	} else if (format == format->Tiff) {
		return L".tif";
	} else {
		return L".bmp";
	}
}

bool breakUp(String ^src, String ^dest, ImageFormat ^format) {
	Bitmap ^b = nullptr;
	try {
		b = gcnew Bitmap(src);
	} catch (Exception ^ex) {
		Console::Error->WriteLine(L"There was a problem opening the source image '"+src+L"': "+ex->Message);
		return false;
	}
	if (b->Width != ANIM_WIDTH || b->Height % ANIM_HEIGHT != 0) {
		Console::Error->WriteLine(L"The source image must be "+ANIM_WIDTH+"px wide and a multiple of "+ANIM_HEIGHT+L"px tall");
		return false;
	}
	if (!Directory::Exists(dest)) {
		try {
			Directory::CreateDirectory(dest);
		} catch (Exception ^ex) {
			Console::Error->WriteLine(L"There was a problem creating the directory '"+dest+L"': "+ex->Message);
			return false;
		}
	}
	String ^name = Path::GetFileNameWithoutExtension(src);
	int frames = b->Height / ANIM_HEIGHT;
	Rectangle r = Rectangle(0, 0, ANIM_WIDTH, ANIM_HEIGHT);
	for (int i = 0; i < frames; i++) {
		r.Y = i*ANIM_HEIGHT;
		String ^path = Path::Combine(dest, name+i.ToString(L"000")+getExt(format));
		Bitmap ^x = b->Clone(r, PixelFormat::Format24bppRgb);
		try {
			x->Save(path, format);
		} catch (Exception ^ex) {
			Console::Error->WriteLine(L"Unable to save '"+path+L"': "+ex->Message);
		}
	}
	return true;
}

Bitmap ^getBitmap(String ^path, bool displayError) {
	Bitmap ^x = nullptr;
	try {
		x = gcnew Bitmap(path);
		if (x->Width != ANIM_WIDTH || x->Height != ANIM_HEIGHT) {
			if (displayError) {
				Console::Error->WriteLine(L"The file '"+path+L"' is the wrong size (needs to be "+ANIM_WIDTH+L"x"+ANIM_HEIGHT+L")");
			}
			x = nullptr;
		}
	} catch (Exception ^ex) {
		if (displayError) {
			Console::Error->WriteLine(L"There was a problem opening the file '"+path+L"': "+ex->Message);
		}
	}
	return x;
}

void SetInt(int x, array<Byte> ^arr, int offset) {
	for (int i = 0; i < 4; i++) {
		arr[offset+i] = (Byte)((x >> (i * 8)) & 0xFF);
    }
}

bool pieceTogether(String ^src, String ^dest, ImageFormat ^format) {
	// Open the directory
	DirectoryInfo ^dir = nullptr;
	try {
		dir = gcnew DirectoryInfo(src);
	} catch (Exception ^ex) {
		Console::Error->WriteLine(L"There was a problem opening the directory '"+src+L"': "+ex->Message);
		return false;
	}

	// Go through each file
	Collections::Generic::List<String^> ^files = gcnew Collections::Generic::List<String^>();
	for each (FileInfo ^fi in dir->GetFiles(L"*"+getExt(format))) {
		if (getBitmap(fi->FullName, true)) {
			files->Add(fi->FullName);
		}
	}
	if (files->Count == 0) {
		Console::Error->WriteLine(L"There were no acceptable files found in '"+src+"'");
		return false;
	}
	if (files->Count != FRAMES) {
		Console::Error->WriteLine(L"Warning: Exepcted "+FRAMES+L" frames but found "+files->Count+L" frames in '"+src+"'");
	}
	
	files->Sort();
	
	unsigned int size = ANIM_WIDTH * files->Count*ANIM_HEIGHT * 3; // bitmap data size
	Bitmap ^b = gcnew Bitmap(ANIM_WIDTH, files->Count*ANIM_HEIGHT, PixelFormat::Format24bppRgb);
	Graphics ^g = Graphics::FromImage(b);
	for (int i = 0; i < files->Count; i++) {
		g->DrawImageUnscaled(getBitmap(files[i], false), 0, i*ANIM_HEIGHT);
	}

	try {
		b->Save(dest, ImageFormat::Bmp);
	} catch (Exception ^ex) {
		Console::Error->WriteLine(L"There was a problem saving to '"+dest+L"': "+ex->Message);
		return false;
	}

	try {
#define HEADER_SIZE 54
		FileStream ^s = File::Open(dest, FileMode::Open, FileAccess::ReadWrite);
		array<Byte> ^x = gcnew array<Byte>(HEADER_SIZE);
		s->Seek(0, SeekOrigin::Begin);
		s->Read(x, 0, HEADER_SIZE);
		//Adjust file size to be +1 of real (this is how the real activity.bmp is)
		SetInt(size+HEADER_SIZE+1, x, 0x02);
		//Add the BMP data size
		SetInt(size, x, 0x22);
		//Set the horiz and vert resolution to 0
		SetInt(0, x, 0x26);
		SetInt(0, x, 0x2A);
		s->Seek(0, SeekOrigin::Begin);
		s->Write(x, 0, HEADER_SIZE);
		s->Close();
		File::SetCreationTimeUtc(dest, DateTime::FromFileTime(creation_time));
		File::SetLastWriteTimeUtc(dest, DateTime::FromFileTime(modification_time));
		File::SetLastAccessTimeUtc(dest, DateTime::FromFileTime(accessed_time));
	} catch (Exception ^ex) {
		Console::Error->WriteLine(L"There was a problem adjusting the saved document '"+dest+L"': "+ex->Message);
		return false;
	}

	return true;
}

int main(array<System::String ^> ^args) {
	Console::WriteLine(L"BootRes Copyright (C) 2010  Jeffrey Bush <jeff@coderforlife.com>");
	Console::WriteLine(L"This program comes with ABSOLUTELY NO WARRANTY;");
	Console::WriteLine(L"This is free software, and you are welcome to redistribute it");
	Console::WriteLine(L"under certain conditions;");
	Console::WriteLine(L"See http://www.gnu.org/licenses/gpl.html for more details.");
	Console::WriteLine();

	if (args->Length != 2 && args->Length != 3) {
		Console::WriteLine(L"Usage: BOOTRES src dest [BMP|PNG|GIF|TIF]");
		Console::WriteLine(L"If src is a file and the dest is a directory, it will break the file up");
		Console::WriteLine(L"If src is a directory and the dest is a file, it will piece the files together");
		Console::WriteLine(L"The last parameter is for file type of pieces, defaults to BMP");
		return 1;
	}

	String ^src = args[0];
	String ^dest = args[1];
	ImageFormat ^format = ImageFormat::Bmp;
	if (args->Length == 3) {
		String ^f = args[2]->ToUpper();
		if (f == L"PNG") {
			format = ImageFormat::Png;
		} else if (f == L"GIF") {
			format = ImageFormat::Png;
		} else if (f == L"TIF") {
			format = ImageFormat::Tiff;
		} else if (f != L"BMP") {
			Console::Error->WriteLine(L"The format for pieces was not understood, defaulting to BMP");
		}
	}
	
	if (Directory::Exists(src)) {
		if (Directory::Exists(dest)) {
			Console::Error->WriteLine(L"Source and destination cannot both be directories.");
			return 1;
		}
		return pieceTogether(src, dest, format) ? 0 : 1;
	} else if (File::Exists(src)) {
		if (File::Exists(dest)) {
			Console::Error->WriteLine(L"Source and destination cannot both be files.");
			return 1;
		}
		return breakUp(src, dest, format) ? 0 : 1;
	} else {
		Console::Error->WriteLine(L"The source does not exist.");
		return 1;
	}
}
